from urllib.parse import urljoin

from pydantic import BaseModel, Field, HttpUrl

from ..client import SDKClient, SDKResponse


class ManifestResponse(BaseModel):
    js: str = Field(alias="app.js")
    css: str = Field(alias="app.css")


class WidgetsService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        self._client = client
        self._url = url

    def get_manifest(self, timeout=3) -> SDKResponse[ManifestResponse]:
        return self._client.get(
            urljoin(str(self._url), "manifest.json"), ManifestResponse, timeout=timeout,
        )



from enum import Enum
from typing import Dict, List, Optional
from urllib.parse import urljoin

from pydantic import BaseModel

from .client import Empty, SDKClient, SDKResponse


class CashierRequest(BaseModel):
    name: str
    inn: str
    user_number: int = 29


class PayMethod(str, Enum):
    CASH = 1
    CASHLESS = 2
    COMBO = 3


class Service(BaseModel):
    name: str
    quantity: int
    department: int
    cost: int


class PayServicesRequest(BaseModel):
    pay_method: PayMethod
    services: List[Service]
    org_name: Optional[str]
    org_inn: Optional[str]
    cash: int
    cashless: int
    phone_email: Optional[str]
    tax: int = 0
    password: int = 29

    class Config:
        use_enum_values = True


class RefundServicesRequest(BaseModel):
    pay_method: PayMethod
    services: List[Service]
    org_name: Optional[str]
    org_inn: Optional[str]
    cash: Optional[int]
    cashless: Optional[int]
    phone_email: Optional[str]
    tax: int = 0
    password: int = 29

    class Config:
        use_enum_values = True


class RegisterResponse(BaseModel):
    code: str
    description: str
    number: Optional[str]  # номер операции в ОФД


class CashRegisterService:
    def __init__(self, client: SDKClient, tokens: Dict):
        self._client = client
        self._tokens = tokens

    def set_cashier(
        self, host: str, query: CashierRequest, timeout: int = 30
    ) -> SDKResponse[RegisterResponse]:
        return self._client.post(
            urljoin(self._get_url(host), "rest/set_cashier/"),
            Empty,
            json=query.dict(),
            headers=self._get_headers(host),
            timeout=timeout,
        )

    def pay_services(
        self, host, query: PayServicesRequest, timeout: int = 30
    ) -> SDKResponse[RegisterResponse]:
        return self._client.post(
            urljoin(self._get_url(host), "rest/multi_service_check/"),
            RegisterResponse,
            json=query.dict(exclude_none=True),
            headers=self._get_headers(host),
            timeout=timeout,
        )

    def refund_services(
        self, host, query: RefundServicesRequest, timeout: int = 30
    ) -> SDKResponse[RegisterResponse]:
        return self._client.post(
            urljoin(self._get_url(host), "rest/multi_service_return_sale/"),
            RegisterResponse,
            json=query.dict(exclude_none=True),
            headers=self._get_headers(host),
            timeout=timeout,
        )

    def report_x(self, host, timeout: int = 30) -> SDKResponse[RegisterResponse]:
        return self._client.post(
            urljoin(self._get_url(host), "rest/x_report/"),
            RegisterResponse,
            json={},
            headers=self._get_headers(host),
            timeout=timeout,
        )

    def report_z(self, host, timeout: int = 30) -> SDKResponse[RegisterResponse]:
        return self._client.post(
            urljoin(self._get_url(host), "rest/z_report/"),
            RegisterResponse,
            json={},
            headers=self._get_headers(host),
            timeout=timeout,
        )

    def cancel(self, host, timeout: int = 30) -> SDKResponse[RegisterResponse]:
        return self._client.post(
            urljoin(self._get_url(host), "rest/cancel_check/"),
            RegisterResponse,
            json={},
            headers=self._get_headers(host),
            timeout=timeout,
        )

    def continue_print(self, host, timeout: int = 30) -> SDKResponse[RegisterResponse]:
        return self._client.post(
            urljoin(self._get_url(host), "rest/continue_print/"),
            RegisterResponse,
            json={},
            headers=self._get_headers(host),
            timeout=timeout,
        )

    def _get_headers(self, host) -> dict:
        return {"Authorization": f"Token {self._tokens[host]}"}

    @staticmethod
    def _get_url(host) -> str:
        return f"http://{host}:8000"



from datetime import date as datetime_date
from enum import Enum, IntEnum
from typing import List, Optional

from pydantic import BaseModel, Field, HttpUrl

from .client import Empty, SDKClient, SDKResponse


COMMON_GIGTEST_TIMEOUT = 10
LONG_GIGTEST_TIMEOUT = 60 + 2


class MedResearchType(IntEnum):
    exam = 0
    lab = 1


class MedResearchBlockType(Enum):
    exam = 0
    lab = 1
    vaccine = 2


class ActivityResponse(BaseModel):
    id: int = Field(description="id")
    key: str = Field(description="unique key")
    parent_id: Optional[int] = Field(description="id of parent activity")
    title: str = Field(description="unique key")


class SectionResponse(BaseModel):
    id: int = Field(description="id")
    activity_key: Optional[str] = Field(
        description="Тип деятельности из activities.key"
    )
    title: str = Field(description="Название")
    image_url: Optional[str] = Field(description="Картинка типа")
    recommended: bool = Field(description="Рекомендовано или нет")


class CountryResponse(BaseModel):
    id: int = Field(description="id")
    name: str = Field(description="Название")


class MedResearchResult(BaseModel):
    id: int = Field(description="id")
    name: str = Field(description="Название")
    key: str = Field(description="Уникальный идентификатор")
    value: int = Field(description="Значение")


class MedicalResearchResponse(BaseModel):
    id: int = Field(description="id")
    name: str = Field(description="Название")
    key: str = Field(description="Уникальный идентификатор")
    activity_keys: List[str] = Field(description="Ключи типов деятельности")
    period: Optional[int] = Field(description="Периодичность в месяцах")
    important: bool = Field(
        description="Важность исследования. В случае положительно результата, "
        "заявитель будет заблокирован"
    )
    type: MedResearchType = Field(description="Тип исследования")
    block_type: MedResearchBlockType = Field(description="Тип группы исследований")
    results: List[MedResearchResult] = Field(
        description="Справочник возможных" " результатов"
    )

    class Config:
        use_enum_values = True


# КЛИЕНТ
class SearchClientRequest(BaseModel):
    medbook_number: str = Field(description="Номер ЛМК")
    lastname: str = Field(description="Фамилия")


class ClientInfo(BaseModel):
    home_address: Optional[str] = Field(description="Адрес")
    phone: Optional[str] = Field(description="Телефон")
    company_name: Optional[str] = Field(description="Название организации")
    position: Optional[str] = Field(description="Должность")
    birthday: Optional[datetime_date] = Field(description="Дата рождения")


class CreateClientRequest(ClientInfo):
    fio: str = Field(description="ФИО")
    birthday: datetime_date = Field(description="Дата рождения")
    country_id: int = Field(description="Id страны по справочнику")


class UpdateClientRequest(ClientInfo):
    pass


class ClientResponse(ClientInfo):
    id: int = Field(description="Id гигтест клиента")
    birthday: Optional[str] = Field(description="Дата рождения")


# МЕДКНИЖКА
class MedicalBookRequest(BaseModel):
    number: str = Field(description="Номер ЛМК")
    regnum: str = Field(description="Рег.номер ЛМК")
    date: datetime_date = Field(description="Дата выдачи ЛМК")
    user_id: int = Field(description="Id гигтест клиента")
    activity_keys: list = Field(description="Ключи типов деятельности")


class UserExtended(ClientInfo):
    fio: str = Field(description="ФИО")
    birth_date: str = Field(description="Дата рождения")


class MedicalBookSearchResponseOther(BaseModel):
    id: int = Field(description="Id ЛМК в гигтесте")
    number: str = Field(description="Номер ЛМК")
    regnum: str = Field(description="Рег. номер ЛМК")
    date: str = Field(description="Дата выдачи ЛМК")
    next_education_date: str = Field(description="Дата следующей аттестации")
    status: str = Field(description="Статус ЛМК: new, accepted, returned, completed")
    activity_names: list = Field(description="Тип деятельности")
    user: UserExtended = Field(description="Гигтест клиент")
    medical_direction_ids: list = Field(description="Гигтест id мед. направлений")


class User(BaseModel):
    fio: str


class MedicalBookSearchResponse(BaseModel):
    number: str = Field(description="Номер ЛМК")
    date: str = Field(description="Дата выдачи ЛМК")
    user: User = Field(description="ФИО гигтест клиента")
    id: int = Field(description="Id ЛМК в гигтесте")


class MedicalBookResponse(BaseModel):
    id: int = Field(description="Id ЛМК в гигтесте")


# АТТЕСТАЦИЯ
class AttestationRequest(BaseModel):
    date: datetime_date = Field(description="Дата первичной/периодической аттестации")
    section_id: int = Field(description="Id раздела")
    questions: dict = Field(description="Объект из id вопрос-ответ")
    medbook_id: Optional[int] = Field(description="Id ЛМК в гигтесте")
    medbook_number: Optional[str] = Field(description="Номер ЛМК")
    mb_regnum: str = Field(description="Рег.номер ЛМК")
    mb_date: datetime_date = Field(description="Дата выдачи ЛМК")
    attestation_number: int = Field(description="1 или 0 (первичная или периодическая)")
    user_id: int = Field(description="Id гигтест клиента")


class AttestationResponse(BaseModel):
    id: int = Field(description="Id аттестации в гигтесте")
    passed: bool = Field(description="Сдал или нет")
    result: str = Field(description="Результат")
    percent: int = Field(description="Процент сдачи")


# МЕДИЦИНА
class MedicineCreateRequest(BaseModel):
    user_id: int = Field(description="Id гигтест клиента")
    medbook_number: str = Field(description="Номер ЛМК")
    activity_key: str = Field(description="Ключи типов деятельности")
    medical_type: str = Field(
        description="Тип медосмотра (preliminary-предварительный, periodic-периодический"
    )
    direction_date: datetime_date = Field(description="Дата направления медосмотра")
    date_completion: Optional[datetime_date] = Field(
        description="Дата завершения медосмотра"
    )


class MedicineUpdateRequest(BaseModel):
    medbook_number: str = Field(description="Номер ЛМК")
    date_completion: datetime_date = Field(description="Дата завершения медосмотра")
    results: dict = Field(description="Исследования")


class MedDirectionResultData(BaseModel):
    result_id: str = Field(description="Id исследования в гигтест")
    date_med_result: datetime_date = Field(description="Дата исследования")
    description: str = Field(description="Описание")
    self_explored: Optional[int] = Field(description="0/1 (проведено у нас или нет)")


class AdditionalMedDirectionResultRequest(MedDirectionResultData):
    medical_research_id: int = Field(description="Id исследования в гигтест")
    medical_direction_id: int = Field(description="Id медобследования в гигтест")


class MedicineCreateOrUpdateResponseResults(BaseModel):
    medical_research_id: int = Field(description="Id исследования в гигтест")
    id: int = Field(description="Id медобследования в гигтест")


class MedicineCreateOrUpdateResponseResultsExtended(
    MedicineCreateOrUpdateResponseResults
):
    medical_research_name: str = Field(description="Название услуги")
    date_med_result: str = Field(description="Дата обследования")
    medical_research_result: int = Field(
        description="Результат обследования. Предположительно 1 - нет противопоказаний"
    )
    medical_research_key: str = Field(description="Код услуги по справочнику гигтест")


class MedicineCreateResponse(BaseModel):
    id: int = Field(description="Id медобследования в гигтест")
    results: List[MedicineCreateOrUpdateResponseResults] = Field(
        description="Исследования"
    )


class MedicineCreateResponseExtended(MedicineCreateResponse):
    results: List[MedicineCreateOrUpdateResponseResultsExtended] = Field(
        description="Исследования"
    )


class MedicineUpdateResponse(BaseModel):
    results: List[MedicineCreateOrUpdateResponseResults] = Field(
        description="Исследования"
    )
    sticker_links: Optional[list] = Field(description="Ссылки на исследования")


class AdditionalMedDirectionResultResponse(BaseModel):
    id: int = Field(description="Id исследования в гигтест")


class GigtestService:
    def __init__(self, client: SDKClient, url: HttpUrl, token: str):
        self._client = client
        self._url = url
        self._token = token
        # СПРАВОЧНИКИ
        self._get_activities_url = "/api/v2/activities"
        self._get_sections_url = "/api/v2/sections"
        self._get_countries_url = "/api/v2/countries"
        self._get_medical_researches_url = "/api/v2/medical-researches"
        # КЛИЕНТ
        self._search_client_url = "/api/v2/users/search"
        self._create_or_update_client_url = "/api/v2/users"
        # МЕДКНИЖКА
        self._medbook_url = "/api/v2/medical-books"
        self._medbook_url_search = "/api/v2/medical-books/search"
        # АТТЕСТАЦИЯ
        self._attestation_url = "/api/v2/protocols"
        # МЕДИЦИНА
        self._create_update_medicine_url = "/api/v2/medical-directions"
        self._additional_medicine_results_url = "/api/v2/medical-research-results"

    # СПРАВОЧНИКИ
    def activities(self) -> SDKResponse[List[ActivityResponse]]:
        return self._client.get(
            self._full_url(self._get_activities_url),
            ActivityResponse,
            timeout=COMMON_GIGTEST_TIMEOUT,
        )

    def sections(self) -> SDKResponse[List[SectionResponse]]:
        return self._client.get(
            self._full_url(self._get_sections_url),
            SectionResponse,
            timeout=COMMON_GIGTEST_TIMEOUT,
        )

    def countries(self) -> SDKResponse[List[CountryResponse]]:
        return self._client.get(
            self._full_url(self._get_countries_url),
            CountryResponse,
            timeout=COMMON_GIGTEST_TIMEOUT,
        )

    def medical_research(self) -> SDKResponse[List[MedicalResearchResponse]]:
        return self._client.get(
            self._full_url(self._get_medical_researches_url),
            MedicalResearchResponse,
            timeout=COMMON_GIGTEST_TIMEOUT,
        )

    # КЛИЕНТ
    def search_client(self, query: SearchClientRequest) -> SDKResponse[ClientResponse]:
        return self._client.get(
            self._full_url(self._search_client_url),
            ClientResponse,
            params=query.dict(),
            timeout=COMMON_GIGTEST_TIMEOUT,
        )

    def create_client(self, query: CreateClientRequest) -> SDKResponse[ClientResponse]:
        return self._client.post(
            self._full_url(self._create_or_update_client_url),
            ClientResponse,
            data=query.json(exclude_none=True),
            timeout=LONG_GIGTEST_TIMEOUT,
            headers={"Content-Type": "application/json"},
        )

    def update_client(
        self, query: UpdateClientRequest, client_gig_test_id: int
    ) -> SDKResponse[ClientResponse]:
        return self._client.put(
            self._full_url(
                self._create_or_update_client_url + "/" + str(client_gig_test_id)
            ),
            ClientResponse,
            data=query.json(exclude_none=True),
            timeout=LONG_GIGTEST_TIMEOUT,
            headers={"Content-Type": "application/json"},
        )

    def get_client(self, client_gig_test_id: int) -> SDKResponse[ClientResponse]:
        return self._client.get(
            self._full_url(
                self._create_or_update_client_url + "/" + str(client_gig_test_id)
            ),
            ClientResponse,
            timeout=COMMON_GIGTEST_TIMEOUT,
        )

    # этот метод для ручного использования!
    def delete_client(self, client_gig_test_id: int) -> SDKResponse[Empty]:
        return self._client.delete(
            self._full_url(
                self._create_or_update_client_url + "/" + str(client_gig_test_id)
            ),
            Empty,
            timeout=LONG_GIGTEST_TIMEOUT,
        )

    # МЕДКНИЖКИ
    def search_medbook_when_other_department(
        self, medbook_number: int
    ) -> SDKResponse[MedicalBookSearchResponseOther]:
        return self._client.get(
            self._full_url(self._medbook_url_search),
            MedicalBookSearchResponseOther,
            params={"number": medbook_number},
            timeout=COMMON_GIGTEST_TIMEOUT,
        )

    def search_medbook(
        self, medbook_number: int
    ) -> SDKResponse[List[MedicalBookSearchResponse]]:
        return self._client.get(
            self._full_url(self._medbook_url),
            MedicalBookSearchResponse,
            params={"MedicalBooksSearch[number]": medbook_number},
            timeout=COMMON_GIGTEST_TIMEOUT,
        )

    def create_medbook(
        self, query: MedicalBookRequest
    ) -> SDKResponse[MedicalBookResponse]:
        return self._client.post(
            self._full_url(self._medbook_url),
            MedicalBookResponse,
            data=query.json(exclude_none=True),
            timeout=LONG_GIGTEST_TIMEOUT,
            headers={"Content-Type": "application/json"},
        )

    def update_medbook(
        self, query: MedicalBookRequest, medbook_gig_test_id: int
    ) -> SDKResponse[MedicalBookResponse]:
        return self._client.put(
            self._full_url(self._medbook_url + "/" + str(medbook_gig_test_id)),
            MedicalBookResponse,
            data=query.json(exclude_none=True),
            timeout=LONG_GIGTEST_TIMEOUT,
            headers={"Content-Type": "application/json"},
        )

    # АТТЕСТАЦИЯ
    def create_attestation(
        self, query: AttestationRequest
    ) -> SDKResponse[AttestationResponse]:
        return self._client.post(
            self._full_url(self._attestation_url),
            AttestationResponse,
            data=query.json(exclude_none=True),
            timeout=LONG_GIGTEST_TIMEOUT,
            headers={"Content-Type": "application/json"},
        )

    def update_attestation(
        self, query: AttestationRequest, gigtest_att_id: int
    ) -> SDKResponse[AttestationResponse]:
        return self._client.put(
            self._full_url(self._attestation_url + "/" + str(gigtest_att_id)),
            AttestationResponse,
            data=query.json(exclude_none=True),
            timeout=LONG_GIGTEST_TIMEOUT,
            headers={"Content-Type": "application/json"},
        )

    # этот метод для ручного использования!
    def delete_attestation(self, gigtest_att_id: int) -> SDKResponse[Empty]:
        return self._client.delete(
            self._full_url(self._attestation_url + "/" + str(gigtest_att_id)),
            Empty,
            timeout=LONG_GIGTEST_TIMEOUT,
        )

    # МЕДИЦИНА
    def get_med_direction(
        self, gigtest_med_direction_id: int
    ) -> SDKResponse[MedicineCreateResponseExtended]:
        return self._client.get(
            self._full_url(
                self._create_update_medicine_url + "/" + str(gigtest_med_direction_id)
            ),
            MedicineCreateResponseExtended,
            timeout=COMMON_GIGTEST_TIMEOUT,
        )

    def create_med_direction(
        self, query: MedicineCreateRequest
    ) -> SDKResponse[MedicineCreateResponse]:
        return self._client.post(
            self._full_url(self._create_update_medicine_url),
            MedicineCreateResponse,
            data=query.json(exclude_none=True),
            timeout=LONG_GIGTEST_TIMEOUT,
            headers={"Content-Type": "application/json"},
        )

    def update_med_direction(
        self, query: MedicineUpdateRequest, gigtest_med_direction_id: int
    ) -> SDKResponse[MedicineUpdateResponse]:
        return self._client.put(
            self._full_url(
                self._create_update_medicine_url + "/" + str(gigtest_med_direction_id)
            ),
            MedicineUpdateResponse,
            data=query.json(exclude_none=True),
            timeout=LONG_GIGTEST_TIMEOUT,
            headers={"Content-Type": "application/json"},
        )

    def delete_med_direction(self, gigtest_med_direction_id: int) -> SDKResponse[Empty]:
        return self._client.delete(
            self._full_url(
                self._create_update_medicine_url + "/" + str(gigtest_med_direction_id)
            ),
            Empty,
            timeout=LONG_GIGTEST_TIMEOUT,
        )

    # TODO выше метод get_med_direction объединить с этим, проверив апи
    def search_med_direction(
        self, gigtest_med_direction_id: int
    ) -> SDKResponse[MedicineUpdateResponse]:
        return self._client.get(
            self._full_url(
                self._create_update_medicine_url + "/" + str(gigtest_med_direction_id)
            ),
            MedicineUpdateResponse,
            timeout=COMMON_GIGTEST_TIMEOUT,
        )

    def delete_med_direction_result(
        self, gigtest_med_direction_result_id: int
    ) -> SDKResponse[Empty]:
        return self._client.delete(
            self._full_url(
                self._additional_medicine_results_url
                + "/"
                + str(gigtest_med_direction_result_id)
            ),
            Empty,
            timeout=LONG_GIGTEST_TIMEOUT,
        )

    def add_additional_result_to_med_direction(
        self, query: AdditionalMedDirectionResultRequest
    ) -> SDKResponse[AdditionalMedDirectionResultResponse]:
        return self._client.post(
            self._full_url(self._additional_medicine_results_url),
            AdditionalMedDirectionResultResponse,
            data=query.json(exclude_none=True),
            timeout=LONG_GIGTEST_TIMEOUT,
            headers={"Content-Type": "application/json"},
        )

    def _full_url(self, url):
        return f"{self._url}{url}?access-token={self._token}"



import datetime
import json

from typing import List

from pydantic import BaseModel, validator

from .client import Empty, SDKClient, SDKResponse


class Cell(BaseModel):
    dt: datetime.date
    time_start: datetime.time
    time_end: datetime.time
    free: bool

    @validator("dt")
    def format_date(cls, v):
        return v.strftime("%Y-%m-%d")

    @validator("time_start", "time_end")
    def format_time(cls, v):
        return v.strftime("%H:%M")


class Doctor(BaseModel):
    id: str
    name: str
    speciality: str
    cells: List[Cell]


class Mc(BaseModel):
    id: str
    name: str
    specialists: List[Doctor]


class SendScheduleRequest(BaseModel):
    med_centers: List[Mc]


class ProDoctorovService:
    def __init__(self, client: SDKClient, login, password):
        self._client = client
        self._login = login
        self._password = password
        self._url = "https://api.prodoctorov.ru/mis"

    def send_schedule(
        self, query: SendScheduleRequest, timeout: int = 10
    ) -> SDKResponse[Empty]:
        cells = {"data": {}}
        for mc in query.med_centers:
            cells[mc.id] = mc.name
            mc_data = cells["data"].setdefault(mc.id, {})
            for spec in mc.specialists:
                mc_data[spec.id] = {
                    "efio": spec.name,
                    "espec": spec.speciality,
                    "cells": [c.dict() for c in spec.cells],
                }
        return self._client.post(
            self._url + "/send_cells/",
            Empty,
            data={
                "login": self._login,
                "password": self._password,
                "cells": json.dumps(cells),
            },
            timeout=timeout,
        )



import datetime

from enum import IntEnum
from typing import List, Optional

from pydantic import BaseModel

from .client import Empty, HttpUrl, SDKClient, SDKResponse


class OrderType(IntEnum):
    lmk = 0
    prof = 1
    lab = 2
    cert = 3


class AddPatientRequest(BaseModel):
    uuid: str
    fname: str
    mname: Optional[str]
    lname: str
    birth: datetime.date
    sex: str

    passport_type: Optional[str]
    passport_number: Optional[str]
    passport_series: Optional[str]
    passport_date: Optional[datetime.date]
    passport_department_code: Optional[str]

    snils: str

    phone: Optional[str]
    email: Optional[str]

    quarter: Optional[str]
    building: Optional[str]
    city: Optional[str]
    street: Optional[str]

    registration_quarter: Optional[str]
    registration_building: Optional[str]
    registration_city: Optional[str]
    registration_street: Optional[str]
    mc_id: int

    @property
    def full_address(self):
        return " ".join(
            a
            for a in [
                self.city,
                self.street,
                self.building,
                self.quarter,
            ]
            if a
        )

    @property
    def registration_address(self):
        return " ".join(
            a
            for a in [
                self.registration_city,
                self.registration_street,
                self.registration_building,
                self.registration_quarter,
            ]
            if a
        )


class MedCenter(BaseModel):
    id: str
    license: str
    license_registration: Optional[str]
    ogrn: str
    okpo: str
    legalname: str
    phone: str  # TODO надо?
    # email: str  # TODO надо?
    legaladdress: str
    zip_code: str


class Order(BaseModel):
    number: str
    exams_start: datetime.datetime
    exams_done: datetime.datetime
    org_name: Optional[str]
    org_inn: Optional[str]
    post: str
    conclusion: str
    next_date: datetime.date
    factors: list


class Doctor(BaseModel):
    speciality: str
    post: str
    mis_id: str
    lname: str
    fname: str
    mname: Optional[str]
    snils: Optional[str]


class Exam(BaseModel):
    conclusion: str
    date: datetime.date
    doctor: Doctor
    id: str


class LabService(BaseModel):
    date: datetime.date
    result: str
    name: str
    id: str


class FuncDiag(BaseModel):
    date: datetime.date
    conclusion: str
    name: str
    id: str


class Vaccine(BaseModel):
    name: str
    step: str
    date: datetime.date


class AddMedRecordRequest(BaseModel):
    mis_id: str  # номер заявки
    order_type: OrderType
    creation_date: datetime.datetime  # lmk - order_time / prof - exams_start
    med_center: MedCenter
    patient: AddPatientRequest
    order: Order
    head_doctor: Optional[Doctor]
    exams: List[Exam]
    lab_services: List[LabService]
    functional_diagnostics: List[FuncDiag]
    vaccinations: List[Vaccine]


class SamdResponse(BaseModel):
    task_id: str


class SamdService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        self._client = client
        self._url = url
        self.add_patient_url = self._url + "/api/patient/"
        self.add_med_record_url = self._url + "/api/med_record/"

    def add_patient(self, query: AddPatientRequest, timeout=3) -> SDKResponse[Empty]:
        return self._client.post(
            self.add_patient_url,
            SamdResponse,
            data=query.json(),
            headers={"Content-Type": "application/json"},
            timeout=timeout,
        )

    def add_med_record(
        self, query: AddMedRecordRequest, timeout=3
    ) -> SDKResponse[Empty]:
        return self._client.post(
            self.add_med_record_url,
            SamdResponse,
            data=query.json(),
            headers={"Content-Type": "application/json"},
            timeout=timeout,
        )



import datetime

from enum import Enum
from typing import List, Optional
from urllib.parse import urljoin

from pydantic import BaseModel, Field, HttpUrl, validator

from ..client import Empty, SDKClient, SDKResponse


class Client(BaseModel):
    phone_number: Optional[str]
    email: Optional[str]
    last_name: str
    first_name: str
    middle_name: Optional[str]
    birth: str
    post: Optional[str] = Field(alias="job")


class Trace(BaseModel):
    uuid: str
    label: str
    dt: Optional[datetime.date]
    description: str

    @validator("dt", pre=True)
    def ignore_time(cls, value):
        if value:
            return datetime.datetime.strptime(value[:10], "%Y-%m-%d").date()
        return


class LmkTraceResponse(BaseModel):
    client: Client
    trace: List[Trace]


class Medicine(BaseModel):
    latest_date: datetime.date
    next_date: Optional[datetime.date]
    nearest_expired_date: Optional[datetime.date]
    has_admission: Optional[bool]


class Attestation(BaseModel):
    latest_date: Optional[datetime.date]
    next_date: Optional[datetime.date]
    reg_date: Optional[datetime.date]


class Lmk(BaseModel):
    reg_number: Optional[str]
    blank_number: str
    med_dat: Optional[datetime.date]
    job: str
    category: str
    type: str
    need_att: bool


class LmkError(str, Enum):
    not_all_medicine = "not_all_medicine"
    attestation_not_found = "attestation_not_found"


class LmkServiceResult(BaseModel):
    name: str
    priority: float
    result_date: Optional[datetime.date]
    expired_date: Optional[datetime.date]
    done: bool
    was: bool
    is_external: bool


class LmkDelivery(BaseModel):
    ready_date: Optional[datetime.date]
    hand_date: Optional[datetime.date]


class Order(BaseModel):
    date: datetime.date
    number: str
    mc: str


class LmkProblem(BaseModel):
    name: str
    code: str
    is_notified: bool


class CheckLmkResponse(BaseModel):
    lmk: Lmk
    medicine: Medicine
    attestation: Optional[Attestation]
    client: Client
    warnings: List[LmkError]
    problems: List[LmkProblem]
    medicine_results: Optional[List[LmkServiceResult]]
    delivery: Optional[LmkDelivery]
    last_order: Optional[Order]

    class Config:
        use_enum_values = True


class OrderTraceRequest(BaseModel):
    phone: str
    lab_number: str


class CheckLmkRequest(BaseModel):
    blank_number: str
    last_name: str
    first_name: str
    take_medicine: bool = False


class SetHandDateRequest(BaseModel):
    uuids: List[str]
    hand_date: datetime.date


class LmkService:
    def __init__(self, client: SDKClient, url: HttpUrl = None):
        self._client = client
        self._url = url

    def order_trace(
        self, query: OrderTraceRequest, timeout=3
    ) -> SDKResponse[LmkTraceResponse]:
        return self._client.get(
            urljoin(str(self._url), "lmk/rest/order_detailed_trace_step/"),
            LmkTraceResponse,
            params=query.dict(),
            timeout=timeout,
        )

    def check_lmk(
        self, query: CheckLmkRequest, timeout=3
    ) -> SDKResponse[CheckLmkResponse]:
        return self._client.get(
            urljoin(str(self._url), "lmk/rest/check_lmk_v2/"),
            CheckLmkResponse,
            params=query.dict(),
            timeout=timeout,
        )

    def set_orders_hand_date(
        self, query: SetHandDateRequest, mis_url, timeout=3
    ) -> SDKResponse[Empty]:
        return self._client.post(
            urljoin(mis_url, "lmk/rest/orders/set_hand_date/"),
            Empty,
            data=query.dict(),
            timeout=timeout,
        )



import json

from enum import Enum
from typing import List, Optional
from urllib.parse import urljoin

from pydantic import BaseModel, HttpUrl

from .client import Empty, SDKClient, SDKResponse


class ResultChoice(str, Enum):
    NORMAL = "NORMAL"
    NOT_NORMAL = "NOT_NORMAL"
    ORDERED = "ORDERED"


class OrderType(str, Enum):
    LMK = "lmk"
    PROF = "prof"


class MisOrderedService(BaseModel):
    service_name: str
    service_code: str
    laboratory: Optional[str]
    result: ResultChoice


class MisOrderRequest(BaseModel):
    external_id: str
    type: OrderType
    ordered_services: List[MisOrderedService]


class LisOrderedService(BaseModel):
    service_name: str
    service_code: str
    result: ResultChoice


class LisOrderRequest(BaseModel):
    external_id: str
    ordered_services: List[LisOrderedService]


class ReestrService:
    def __init__(self, client: SDKClient, url: HttpUrl, token: str):
        self._client = client
        self._url = url
        assert self._url.endswith("/")
        self._token = token

    def send_mis_order(self, query: MisOrderRequest, timeout=3) -> SDKResponse[Empty]:
        return self._client.post(
            urljoin(str(self._url), "api/v1/mis_order"),
            Empty,
            data=json.dumps(query.dict()),
            timeout=timeout,
            headers={"Authorization": f"Bearer {self._token}"},
        )

    def send_lis_order(self, query: LisOrderRequest, timeout=3) -> SDKResponse[Empty]:
        return self._client.post(
            urljoin(str(self._url), "api/v1/lab_order"),
            Empty,
            data=json.dumps(query.dict()),
            timeout=timeout,
            headers={"Authorization": f"Bearer {self._token}"},
        )
